﻿using System;
using System.Collections;
using System.Linq;
using System.Text;
using System.ComponentModel;

    public class DataGridCollection : CollectionBase, IBindingList
    {
        public delegate object CreateNewItemDelegate();
        public delegate void RemoveItemDelegate(object item);

        #region private fields

        private CreateNewItemDelegate createNewItem;
        private RemoveItemDelegate removeItem;
        private ListChangedEventHandler onListChanged;

        #endregion

        public DataGridCollection(IList items, CreateNewItemDelegate createNewItem,
                                              RemoveItemDelegate removeItem)
        {
            foreach (object i in items)
            {
                ((IList)this).Add(i);
            }
            if (items.Count == 0)
            {
                object o = createNewItem();
                ((IList)this).Add(o);
            }
            this.createNewItem = createNewItem;
            this.removeItem = removeItem;
        }

        public void AddItem(object value)
        {
            List.Add(value);
        }

        #region Overriden

        protected virtual void OnListChanged(ListChangedEventArgs ev)
        {
            if (onListChanged != null)
            {
                onListChanged(this, ev);
            }
        }

        protected override void OnClearComplete()
        {
            OnListChanged(new ListChangedEventArgs(ListChangedType.Reset, -1));
        }

        protected override void OnInsertComplete(int index, object value)
        {
            OnListChanged(new ListChangedEventArgs(ListChangedType.ItemAdded, index));
        }

        protected override void OnRemoveComplete(int index, object value)
        {
            removeItem(value);
            OnListChanged(new ListChangedEventArgs(ListChangedType.ItemDeleted, index));
        }

        protected override void OnSetComplete(int index, object oldValue, object newValue)
        {
            if (oldValue != newValue)
            {
                OnListChanged(new ListChangedEventArgs(ListChangedType.ItemAdded, index));
            }
        }

        #endregion

        #region IBindingList Members

        public object AddNew()
        {
            object newItem = createNewItem();
            List.Add(newItem);
            return newItem;
        }

        public bool AllowEdit
        {
            get { return true; }
        }

        public bool AllowNew
        {
            get { return true; }
        }

        public bool AllowRemove
        {
            get { return true; }
        }

        public bool SupportsChangeNotification
        {
            get { return true; }
        }

        public bool SupportsSearching
        {
            get { return false; }
        }

        public bool SupportsSorting
        {
            get { return false; }
        }

        public event ListChangedEventHandler ListChanged
        {
            add
            {
                onListChanged += value;
            }
            remove
            {
                onListChanged -= value;
            }
        }

        #region Unsupported IBindingList members

        public void AddIndex(PropertyDescriptor property)
        {
            throw new NotImplementedException();
        }

        public void ApplySort(PropertyDescriptor property, ListSortDirection direction)
        {
            throw new NotImplementedException();
        }

        public int Find(PropertyDescriptor property, object key)
        {
            throw new NotImplementedException();
        }

        public bool IsSorted
        {
            get { throw new NotImplementedException(); }
        }
        public void RemoveIndex(PropertyDescriptor property)
        {
            throw new NotImplementedException();
        }

        public void RemoveSort()
        {
            throw new NotImplementedException();
        }

        public ListSortDirection SortDirection
        {
            get { throw new NotImplementedException(); }
        }

        public PropertyDescriptor SortProperty
        {
            get { throw new NotImplementedException(); }
        }

        #endregion

        #endregion
    }

